@@ -27,7 +27,6 @@
 
 #include "compressionoptionswidget.h"
 #include "ark_debug.h"
-#include "archiveformat.h"
 #include "pluginmanager.h"
 #include "settings.h"
 
@@ -94,7 +93,7 @@ ulong CompressionOptionsWidget::volumeSize() const
 {
     if (collapsibleMultiVolume->isEnabled() && multiVolumeCheckbox->isChecked()) {
         // Convert to kilobytes.
-        return volumeSizeSpinbox->value() * 1024;
+        return static_cast<ulong>(volumeSizeSpinbox->value()) * 1024;
     } else {
         return 0;
     }
@@ -110,10 +109,15 @@ QString CompressionOptionsWidget::password() const
     return pwdWidget->password();
 }
 
-void CompressionOptionsWidget::updateWidgets()
+ArchiveFormat CompressionOptionsWidget::archiveFormat() const
 {
     const KPluginMetaData metadata = PluginManager().preferredPluginFor(m_mimetype)->metaData();
-    const ArchiveFormat archiveFormat = ArchiveFormat::fromMetadata(m_mimetype, metadata);
+    return ArchiveFormat::fromMetadata(m_mimetype, metadata);
+}
+
+void CompressionOptionsWidget::updateWidgets()
+{
+    const ArchiveFormat archiveFormat = this->archiveFormat();
     Q_ASSERT(archiveFormat.isValid());
 
     if (archiveFormat.encryptionType() != Archive::Unencrypted) {
@@ -283,7 +287,18 @@ void CompressionOptionsWidget::slotCompMethodChanged(const QString &value)
         } else {
             encMethodComboBox->insertItem(0, QStringLiteral("AES256"));
         }
+    }
 
+    const ArchiveFormat archiveFormat = this->archiveFormat();
+    Q_ASSERT(archiveFormat.isValid());
+
+    if (m_mimetype == QMimeDatabase().mimeTypeForName(QStringLiteral("application/zip"))) {
+
+        if (value == QLatin1String("Zstd")) {
+            compLevelSlider->setMaximum(22);
+        } else {
+            compLevelSlider->setMaximum(archiveFormat.maxCompressionLevel());
+        }
     }
 }
 
