@@ -33,6 +33,7 @@
 #include <KMessageWidget>
 
 #include <QModelIndex>
+#include <QTimer>
 
 class ArchiveModel;
 class ArchiveSortFilterModel;
@@ -67,7 +68,13 @@ public:
         OpenFileWith
     };
 
-    Part(QWidget *parentWidget, QObject *parent, const QVariantList &);
+    enum OverwriteBehaviour {
+        ShowOverwriteDialog,
+        DoNotShowOverwriteDialog
+    };
+
+
+    Part(QWidget *parentWidget, QObject *parent, const KPluginMetaData &metaData, const QVariantList &);
     ~Part() override;
 
     bool openFile() override;
@@ -78,6 +85,11 @@ public:
     QList<Kerfuffle::SettingsPage*> settingsPages(QWidget *parent) const override;
     bool eventFilter(QObject *target, QEvent *event) override;
 
+    /**
+     * Return custom compnentName for KXMLGUIClient, as by history not the plugin id is used
+     */
+    QString componentName() const override;
+
     /**
      * Validate the localFilePath() associated to this part.
      * If the file is not valid, an error message is displayed to the user.
@@ -119,8 +131,9 @@ private Q_SLOTS:
      * or drag'n'drop event, for adding a watched file it has empty.
      * @param relPath Relative path of watched entry inside the archive. Is used only for adding temporarily extracted
      * watched file.
+     * @param onOverwrite Whether to show a confirmation dialog when files will be overwritten
      */
-    void slotAddFiles(const QStringList &files, const Kerfuffle::Archive::Entry *destination, const QString &relPath);
+    void slotAddFiles(const QStringList &files, const Kerfuffle::Archive::Entry *destination, const QString &relPath, Ark::Part::OverwriteBehaviour onOverwrite = ShowOverwriteDialog);
     void slotDroppedFiles(const QStringList &files, const Kerfuffle::Archive::Entry *destination);
 
     /**
@@ -154,6 +167,7 @@ private Q_SLOTS:
     void setBusyGui();
     void setReadyGui();
     void setFileNameFromArchive();
+    void slotResetFileChangeTimer(const QString& file);
     void slotWatchedFileModified(const QString& file);
     void slotShowComment();
     void slotAddComment();
@@ -223,10 +237,13 @@ private:
     Kerfuffle::Archive::Entry *m_destination;
     QModelIndexList m_cutIndexes;
 
+    QTimer m_watchedFileChangeTimer;
+    QString m_lastChangedFilename;
+
     KAbstractWidgetJobTracker  *m_jobTracker;
     KParts::StatusBarExtension *m_statusBarExtension;
     QVBoxLayout *m_vlayout;
-    QFileSystemWatcher *m_fileWatcher;
+    std::unique_ptr<QFileSystemWatcher> m_fileWatcher;
     QSplitter *m_commentSplitter;
     QGroupBox *m_commentBox;
     QPlainTextEdit *m_commentView;
